"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.deactivate = exports.activate = void 0;
const vscode_1 = require("vscode");
const vscode_languageclient_1 = require("vscode-languageclient");
const EncodedTSCFileSystemProvider_1 = require("./EncodedTSCFileSystemProvider");
const langserver_1 = require("./langserver");
let client;
let extensionDir;
const isEncodedTSC = (data) => {
    const commandRegex = /<[A-Z-+0-9]{3}/g;
    const eventIDRegex = /#[0-9]{4}/g;
    return commandRegex.test(data) && eventIDRegex.test(data);
};
const getRcUri = () => vscode_1.workspace.workspaceFolders && vscode_1.Uri.joinPath(vscode_1.workspace.workspaceFolders[0].uri, '.tscrc.json');
const sendConfigUpdateRequest = () => {
    const uri = getRcUri();
    if (!uri) {
        return;
    }
    vscode_1.workspace.fs.readFile(uri)
        .then(buffer => {
        const contents = buffer.toString();
        try {
            const json = JSON.parse(contents);
            client.sendRequest('tsc/setConfig', json);
        }
        catch (_a) {
            console.log('Local .tscrc.json is invalid. Using default configuration.');
        }
    }, function () {
        console.log('Local .tscrc.json does not exist. Using default configuration.');
    });
};
const sendConfigResetRequest = () => client.sendRequest('tsc/resetConfig');
function activate(ctx) {
    return __awaiter(this, void 0, void 0, function* () {
        extensionDir = ctx.extensionPath;
        const lspCommand = yield langserver_1.getLanguageServerPath(extensionDir);
        const executable = {
            command: lspCommand,
            args: ['start']
        };
        const serverOptions = {
            run: executable,
            debug: executable
        };
        const clientOptions = {
            documentSelector: [
                {
                    scheme: 'file',
                    language: 'tsc'
                },
                {
                    scheme: 'encoded-tsc',
                    language: 'tsc'
                },
                {
                    scheme: 'untitled',
                    language: 'tsc'
                }
            ]
        };
        vscode_1.workspace.registerFileSystemProvider('encoded-tsc', new EncodedTSCFileSystemProvider_1.EncodedTSCFileSystemProvider(), {
            isCaseSensitive: process.platform === 'linux'
        });
        client = new vscode_languageclient_1.LanguageClient('tsc', 'Cave Story TSC', serverOptions, clientOptions);
        client.start();
        client.onReady().then(sendConfigUpdateRequest);
        vscode_1.workspace.onDidRenameFiles(e => {
            var _a, _b;
            const rcUri = (_b = (_a = getRcUri()) === null || _a === void 0 ? void 0 : _a.toString()) !== null && _b !== void 0 ? _b : '';
            if (e.files.find(f => f.newUri.toString() === rcUri)) {
                sendConfigUpdateRequest();
            }
            if (e.files.find(f => f.oldUri.toString() === rcUri)) {
                sendConfigResetRequest();
            }
        });
        vscode_1.workspace.onDidDeleteFiles(e => {
            var _a, _b;
            const rcUri = (_b = (_a = getRcUri()) === null || _a === void 0 ? void 0 : _a.toString()) !== null && _b !== void 0 ? _b : '';
            if (rcUri && e.files.map(f => f.toString()).includes(rcUri)) {
                sendConfigResetRequest();
            }
        });
        vscode_1.workspace.onDidSaveTextDocument(e => {
            var _a, _b;
            const rcUri = (_b = (_a = getRcUri()) === null || _a === void 0 ? void 0 : _a.toString()) !== null && _b !== void 0 ? _b : '';
            if (e.uri.toString() === rcUri) {
                sendConfigUpdateRequest();
            }
        });
        vscode_1.workspace.onDidOpenTextDocument(e => {
            if (e.languageId === 'tsc' && e.uri.scheme !== 'encoded-tsc') {
                const reopenAction = 'Reopen in text view';
                // Check if the .tsc file is encoded.
                const startPos = e.positionAt(0);
                const endPos = e.positionAt(512);
                const tscData = e.getText(new vscode_1.Range(startPos, endPos));
                if (isEncodedTSC(tscData)) {
                    return;
                }
                vscode_1.window.showInformationMessage('Encrypted TSC found, do you want to reopen it in decoded mode (the file will be still saved as encrypted)?', reopenAction).then((result) => __awaiter(this, void 0, void 0, function* () {
                    if (result !== reopenAction) {
                        return;
                    }
                    const uri = vscode_1.Uri.file(e.fileName).with({ scheme: 'encoded-tsc' });
                    try {
                        const doc = yield vscode_1.workspace.openTextDocument(uri);
                        yield vscode_1.window.showTextDocument(doc, { preview: false });
                    }
                    catch (e) {
                        console.error(e);
                    }
                }));
            }
        });
        ctx.subscriptions.push(vscode_1.commands.registerCommand('vscode-tsc.restartLsp', () => __awaiter(this, void 0, void 0, function* () {
            if (!client) {
                // should never fire but we're checking anyway
                return;
            }
            yield client.stop();
            yield client.start();
            sendConfigUpdateRequest();
        })));
    });
}
exports.activate = activate;
function deactivate() {
    if (!client) {
        return undefined;
    }
    return client.stop();
}
exports.deactivate = deactivate;
//# sourceMappingURL=extension.js.map