"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getLanguageServerPath = exports.downloadBinary = exports.removeOldBinary = exports.getLatestReleaseInfo = exports.getCurrentVersion = void 0;
const node_fetch_1 = require("node-fetch");
const util_1 = require("util");
const child_process_1 = require("child_process");
const fs = require("fs");
const mkdirp = require("mkdirp");
const lookpath_1 = require("lookpath");
const semver_1 = require("semver");
const vscode_1 = require("vscode");
const integrity_1 = require("./utils/integrity");
const execAsync = util_1.promisify(child_process_1.exec);
const headers = {
    // eslint-disable-next-line
    Accept: 'application/json',
    // eslint-disable-next-line
    'Content-Type': 'application/json'
};
function getCurrentVersion(binaryPath) {
    return __awaiter(this, void 0, void 0, function* () {
        const versionString = yield execAsync(`${binaryPath} --version`);
        return versionString.stdout.split(' ').pop();
    });
}
exports.getCurrentVersion = getCurrentVersion;
function getLatestReleaseInfo() {
    return __awaiter(this, void 0, void 0, function* () {
        const res = yield node_fetch_1.default('https://api.github.com/repos/nimblebun/tsc-language-server/releases/latest', { headers })
            .then(res => res.json());
        const changelogUrl = res.html_url;
        const version = res.name;
        let arch = process.arch;
        if (arch === 'x64') {
            arch = 'amd64';
        }
        if (arch === 'ia32') {
            arch = '386';
        }
        let platform = process.platform.toString();
        if (platform === 'win32') {
            platform = 'windows';
            arch = `${arch}.exe`; // saving an extra op here
        }
        const checksums = res.assets.find((a) => a.name === `tsc-ls_${version.slice(1)}_SHA256SUM`);
        const binary = res.assets.find((a) => a.name === `tsc-ls_${version.slice(1)}_${platform}_${arch}`);
        if (!checksums) {
            throw new Error('Failed to download checksums to check file integrity.');
        }
        if (!binary) {
            throw new Error('No binary releases for your OS/arch.');
        }
        const checksumsSignatureUrl = `${checksums.browser_download_url}.sig`;
        return {
            name: binary.name,
            changelogUrl,
            version,
            checksumsUrl: checksums.browser_download_url,
            checksumsSignatureUrl,
            binaryUrl: binary.browser_download_url
        };
    });
}
exports.getLatestReleaseInfo = getLatestReleaseInfo;
;
function removeOldBinary(binaryPath) {
    if (!fs.existsSync(binaryPath)) {
        return;
    }
    fs.unlinkSync(binaryPath);
}
exports.removeOldBinary = removeOldBinary;
;
function downloadBinary(release, targetPath) {
    return __awaiter(this, void 0, void 0, function* () {
        const res = yield node_fetch_1.default(release.binaryUrl);
        return new Promise((resolve, reject) => {
            res.body.pipe(fs.createWriteStream(targetPath));
            res.body.on('error', reject);
            res.body.on('end', resolve);
        });
    });
}
exports.downloadBinary = downloadBinary;
;
function install(release, targetPath) {
    return __awaiter(this, void 0, void 0, function* () {
        return new Promise(resolve => {
            removeOldBinary(targetPath);
            vscode_1.window.withProgress({
                cancellable: true,
                location: vscode_1.ProgressLocation.Notification,
                title: 'Installing TSC language server',
            }, (progress, _) => __awaiter(this, void 0, void 0, function* () {
                progress.report({ message: 'Downloading binary...', increment: 40 });
                yield downloadBinary(release, targetPath);
                progress.report({ message: 'Verifying integrity...', increment: 40 });
                yield integrity_1.default(release, targetPath);
                fs.chmodSync(targetPath, 0o700);
                return resolve();
            }));
        });
    });
}
function getLanguageServerPath(extensionDir) {
    return __awaiter(this, void 0, void 0, function* () {
        if (yield lookpath_1.lookpath('tsc-ls')) {
            return 'tsc-ls';
        }
        yield mkdirp(`${extensionDir}/lsp`);
        let localServer = `${extensionDir}/lsp/tsc-ls`;
        if (process.platform.toString() === 'win32') {
            localServer = `${localServer}.exe`;
        }
        const isInstalled = fs.existsSync(localServer);
        let doUpdate = false;
        const latestRelease = yield getLatestReleaseInfo();
        if (isInstalled) {
            const currentVersion = yield getCurrentVersion(localServer);
            if (!currentVersion) {
                // something happened, try to return the path anyway
                return localServer;
            }
            if (semver_1.gt(latestRelease.version, currentVersion)) {
                const choice = yield vscode_1.window.showInformationMessage(`Update available for the TSC language server (${latestRelease.version}). Do you want to install it?`, 'Yes', 'No');
                if (choice === 'No') {
                    return localServer;
                }
                doUpdate = true;
            }
            if (!doUpdate) {
                return localServer;
            }
        }
        try {
            yield install(latestRelease, localServer);
        }
        catch (err) {
            vscode_1.window.showErrorMessage(`Failed to install tsc-ls: ${err}.`);
        }
        vscode_1.window.showInformationMessage(`Installed TSC Language Server ${latestRelease.version}.`, 'View Changelog')
            .then(choice => {
            if (choice === 'View Changelog') {
                vscode_1.env.openExternal(vscode_1.Uri.parse(latestRelease.changelogUrl));
            }
        });
        return localServer;
    });
}
exports.getLanguageServerPath = getLanguageServerPath;
;
//# sourceMappingURL=langserver.js.map